@@ -1,3 +1,4 @@
+#![cfg_attr(not(feature = "std"), no_std)]
 #![deny(
     anonymous_parameters,
     clippy::all,
@@ -34,12 +35,12 @@
 )]
 #![doc(test(attr(deny(warnings))))]
 
-use core::convert::{TryFrom, TryInto};
+use core::borrow::Borrow;
+use core::cmp::Ordering;
 use core::fmt;
+#[cfg(feature = "std")]
 use std::error::Error;
 
-pub use deranged_macros::ranged_int;
-
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct TryFromIntError;
 
@@ -48,6 +49,7 @@ impl fmt::Display for TryFromIntError {
         f.write_str("out of range integral type conversion attempted")
     }
 }
+#[cfg(feature = "std")]
 impl Error for TryFromIntError {}
 
 macro_rules! const_try_opt {
@@ -60,20 +62,37 @@ macro_rules! const_try_opt {
 }
 
 macro_rules! if_signed {
-    (signed) => {};
+    (true $($x:tt)*) => { $($x)*};
+    (false $($x:tt)*) => {};
+}
+
+macro_rules! article {
+    (true) => {
+        "An"
+    };
+    (false) => {
+        "A"
+    };
 }
 
 macro_rules! impl_ranged {
     ($(
-        $([$maybe_signed:ident])? $type:ident($internal:ident): {
+        $type:ident {
+            internal: $internal:ident
+            signed: $is_signed:ident
             into: [$($into:ident),* $(,)?]
             try_into: [$($try_into:ident),* $(,)?]
             try_from: [$($try_from:ident),* $(,)?]
         }
     )*) => {$(
-        /// An integer that is known to be in the range `MIN..=MAX`.
+        #[doc = concat!(
+            article!($is_signed),
+            " `",
+            stringify!($internal),
+            "` that is known to be in the range `MIN..=MAX`.",
+        )]
         #[repr(transparent)]
-        #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+        #[derive(Clone, Copy, Eq, Ord, Hash)]
         pub struct $type<const MIN: $internal, const MAX: $internal>(
             $internal,
         );
@@ -191,13 +210,13 @@ macro_rules! impl_ranged {
                 Self::new(const_try_opt!(self.0.checked_shr(rhs)))
             }
 
-            $(if_signed!($maybe_signed);
+            if_signed!($is_signed
             /// Checked absolute value. Computes `self.abs()`, returning `None`
             /// if the resulting value is out of range.
             #[must_use = "this returns the result of the operation, without modifying the original"]
             pub const fn checked_abs(self) -> Option<Self> {
                 Self::new(const_try_opt!(self.0.checked_abs()))
-            })?
+            });
 
             /// Checked exponentiation. Computes `self.pow(exp)`, returning
             /// `None` if the resulting value is out of range.
@@ -220,21 +239,21 @@ macro_rules! impl_ranged {
                 Self::new_saturating(self.0.saturating_sub(rhs))
             }
 
-            $(if_signed!($maybe_signed);
+            if_signed!($is_signed
             /// Saturating integer negation. Computes `self - rhs`, saturating
             /// at the numeric bounds.
             #[must_use = "this returns the result of the operation, without modifying the original"]
             pub const fn saturating_neg(self) -> Self {
                 Self::new_saturating(self.0.saturating_neg())
-            })?
+            });
 
-            $(if_signed!($maybe_signed);
+            if_signed!($is_signed
             /// Saturating absolute value. Computes `self.abs()`, saturating at
             /// the numeric bounds.
             #[must_use = "this returns the result of the operation, without modifying the original"]
             pub const fn saturating_abs(self) -> Self {
                 Self::new_saturating(self.0.saturating_abs())
-            })?
+            });
 
             /// Saturating integer multiplication. Computes `self * rhs`,
             /// saturating at the numeric bounds.
@@ -263,6 +282,29 @@ macro_rules! impl_ranged {
             }
         }
 
+        impl<const MIN: $internal, const MAX: $internal> AsRef<$internal> for $type<MIN, MAX> {
+            fn as_ref(&self) -> &$internal {
+                &self.0
+            }
+        }
+
+        impl<const MIN: $internal, const MAX: $internal> Borrow<$internal> for $type<MIN, MAX> {
+            fn borrow(&self) -> &$internal {
+                &self.0
+            }
+        }
+
+        impl<
+            const MIN_A: $internal,
+            const MAX_A: $internal,
+            const MIN_B: $internal,
+            const MAX_B: $internal,
+        > PartialEq<$type<MIN_B, MAX_B>> for $type<MIN_A, MAX_A> {
+            fn eq(&self, other: &$type<MIN_B, MAX_B>) -> bool {
+                self.0 == other.0
+            }
+        }
+
         impl<const MIN: $internal, const MAX: $internal> PartialEq<$internal> for $type<MIN, MAX> {
             fn eq(&self, other: &$internal) -> bool {
                 self.0 == *other
@@ -275,6 +317,29 @@ macro_rules! impl_ranged {
             }
         }
 
+        impl<
+            const MIN_A: $internal,
+            const MAX_A: $internal,
+            const MIN_B: $internal,
+            const MAX_B: $internal,
+        > PartialOrd<$type<MIN_B, MAX_B>> for $type<MIN_A, MAX_A> {
+            fn partial_cmp(&self, other: &$type<MIN_B, MAX_B>) -> Option<Ordering> {
+                self.0.partial_cmp(&other.0)
+            }
+        }
+
+        impl<const MIN: $internal, const MAX: $internal> PartialOrd<$internal> for $type<MIN, MAX> {
+            fn partial_cmp(&self, other: &$internal) -> Option<Ordering> {
+                self.0.partial_cmp(other)
+            }
+        }
+
+        impl<const MIN: $internal, const MAX: $internal> PartialOrd<$type<MIN, MAX>> for $internal {
+            fn partial_cmp(&self, other: &$type<MIN, MAX>) -> Option<Ordering> {
+                self.partial_cmp(&other.0)
+            }
+        }
+
         impl<const MIN: $internal, const MAX: $internal> fmt::Binary for $type<MIN, MAX> {
             fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                 self.0.fmt(f)
@@ -321,11 +386,7 @@ macro_rules! impl_ranged {
             type Error = TryFromIntError;
 
             fn try_from(value: $type<MIN, MAX>) -> Result<Self, Self::Error> {
-                if (MIN..=MAX).contains(&value.0) {
-                    Ok(value.try_into()?)
-                } else {
-                    Err(TryFromIntError)
-                }
+                value.0.try_into().map_err(|_| TryFromIntError)
             }
         })*
 
@@ -342,64 +403,159 @@ macro_rules! impl_ranged {
                     Err(TryFromIntError)
                 } else {
                     match TryFrom::try_from(value) {
-                        Ok(value) => Ok(value),
+                        Ok(value) => Ok(Self(value)),
                         Err(_) => Err(TryFromIntError),
                     }
                 }
             }
         })*
+
+        #[cfg(feature = "serde")]
+        impl<const MIN: $internal, const MAX: $internal> serde::Serialize for $type<MIN, MAX> {
+            fn serialize<S: serde::Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+                self.get().serialize(serializer)
+            }
+        }
+
+        #[cfg(feature = "serde")]
+        impl<
+            'de,
+            const MIN: $internal,
+            const MAX: $internal,
+        > serde::Deserialize<'de> for $type<MIN, MAX> {
+            fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+                let internal = <$internal>::deserialize(deserializer)?;
+                Self::new(internal).ok_or_else(|| <D::Error as serde::de::Error>::invalid_value(
+                    serde::de::Unexpected::Other("integer"),
+                    &format!("an integer in the range {}..={}", MIN, MAX).as_ref()
+                ))
+            }
+        }
+
+        #[cfg(feature = "rand")]
+        impl<
+            const MIN: $internal,
+            const MAX: $internal,
+        > rand::distributions::Distribution<$type<MIN, MAX>> for rand::distributions::Standard {
+            fn sample<R: rand::Rng + ?Sized>(&self, rng: &mut R) -> $type<MIN, MAX> {
+                $type(rng.gen_range(MIN..=MAX))
+            }
+        }
     )*};
 }
 
 impl_ranged! {
-    RangedU8(u8): {
-        into: [u8, u16, u32, u64, u128, i16, i32, i64, i128]
+    U8 {
+        internal: u8
+        signed: false
+        into: [u8, u16, u32, u64, u128, usize, i16, i32, i64, i128, isize]
         try_into: [i8]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    RangedU16(u16): {
-        into: [u16, u32, u64, u128, i32, i64, i128]
-        try_into: [u8, i8, i16]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+    U16 {
+        internal: u16
+        signed: false
+        into: [u16, u32, u64, u128, usize, i32, i64, i128]
+        try_into: [u8, i8, i16, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    RangedU32(u32): {
+    U32 {
+        internal: u32
+        signed: false
         into: [u32, u64, u128, i64, i128]
-        try_into: [u8, u16, i8, i16, i32]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, usize, i8, i16, i32, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    RangedU64(u64): {
+    U64 {
+        internal: u64
+        signed: false
         into: [u64, u128, i128]
-        try_into: [u8, u16, u32, i8, i16, i32, i64]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, u32, usize, i8, i16, i32, i64, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    RangedU128(u128): {
+    U128 {
+        internal: u128
+        signed: false
         into: [u128]
-        try_into: [u8, u16, u32, u64, i8, i16, i32, i64, i128]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, u32, u64, usize, i8, i16, i32, i64, i128, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    [signed] RangedI8(i8): {
-        into: [i8, i16, i32, i64, i128]
-        try_into: [u8, u16, u32, u64, u128]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+    Usize {
+        internal: usize
+        signed: false
+        into: [usize]
+        try_into: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    [signed] RangedI16(i16): {
-        into: [i16, i32, i64, i128]
-        try_into: [u8, u16, u32, u64, u128, i8]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+    I8 {
+        internal: i8
+        signed: true
+        into: [i8, i16, i32, i64, i128, isize]
+        try_into: [u8, u16, u32, u64, u128, usize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    [signed] RangedI32(i32): {
+    I16 {
+        internal: i16
+        signed: true
+        into: [i16, i32, i64, i128, isize]
+        try_into: [u8, u16, u32, u64, u128, usize, i8]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
+    }
+    I32 {
+        internal: i32
+        signed: true
         into: [i32, i64, i128]
-        try_into: [u8, u16, u32, u64, u128, i8, i16]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, u32, u64, u128, usize, i8, i16, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    [signed] RangedI64(i64): {
+    I64 {
+        internal: i64
+        signed: true
         into: [i64, i128]
-        try_into: [u8, u16, u32, u64, u128, i8, i16, i32]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, u32, u64, u128, usize, i8, i16, i32, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
     }
-    [signed] RangedI128(i128): {
+    I128 {
+        internal: i128
+        signed: true
         into: [i128]
-        try_into: [u8, u16, u32, u64, u128, i8, i16, i32, i64]
-        try_from: [u8, u16, u32, u64, u128, i8, i16, i32, i64, i128]
+        try_into: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, isize]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
+    }
+    Isize {
+        internal: isize
+        signed: true
+        into: [isize]
+        try_into: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128]
+        try_from: [u8, u16, u32, u64, u128, usize, i8, i16, i32, i64, i128, isize]
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use std::convert::TryFrom;
+
+    use crate::{TryFromIntError, U32};
+
+    #[test]
+    fn test_try_from_primitive_to_deranged() {
+        assert_eq!(U32::<100, 200>::try_from(50), Err(TryFromIntError));
+        assert_eq!(U32::<100, 200>::try_from(100), Ok(U32(100)));
+        assert_eq!(U32::<100, 200>::try_from(150), Ok(U32(150)));
+        assert_eq!(U32::<100, 200>::try_from(200), Ok(U32(200)));
+        assert_eq!(U32::<100, 200>::try_from(250), Err(TryFromIntError));
+    }
+
+    #[test]
+    #[allow(clippy::unwrap_used)]
+    fn test_try_from_deranged_to_primitive() {
+        assert_eq!(
+            u32::try_from(U32::<1000, 2000>::new(1500).unwrap()),
+            Ok(1500)
+        );
+        assert_eq!(
+            u8::try_from(U32::<1000, 2000>::new(1500).unwrap()),
+            Err(TryFromIntError)
+        );
     }
 }
